//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Class          : GUIPrompt
//
// Author         : Richard E. Pattis
//                  Computer Science Department
//                  Carnegie Mellon University
//                  5000 Forbes Avenue
//                  Pittsburgh, PA 15213-3891
//                  e-mail: pattis@cs.cmu.edu
//
// Maintainer     : Author
//
//
// Description:
//
//   The Prompt class is a (static) method library supplying simple methods
// for prompting the user (in pop-up windows) for various types of
// information. All these methods reprompt the user if the response to the
// prompt is considered "illegal" (e.g., not the right type, not in range).
//
// Future Plans   : Low/High for Double, Default values for all
//
// Program History:
//   8/20/01: R. Pattis - Operational for 15-100
//
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

package edu.cmu.cs.pattis.cs151xx;



import java.awt.Frame;
import java.awt.Toolkit;
import java.awt.Dimension;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JOptionPane;
import javax.swing.BoxLayout;
import javax.swing.JDialog;
import javax.swing.JLabel;

import java.beans.*; 			    //Contains property change stuff

import java.math.BigInteger;  //For BigInteger constructor





/** 
 * The <code>Prompt</code> class is a (static) method library supplying
 *   simple methods for prompting the user (in pop-up windows) for
 *   various types of information.
 * <p>
 * All these methods reprompt the user if the response to the prompt
 *   is considered "illegal" (e.g., not the right type).
 * 
 * @author Richard E. Pattis (Computer Science Department, Carnegie Mellon)
*/
public class GUIPrompt {



	/** 
	 * Returns the user's <code>int</code> response to the prompt message.
	 * 
	 * @param  message specifies the prompt displayed to the user
	 *
	 * @return the value entered by the user
	*/
  public static int forInt (String message)
  {
	  ValidatorTranslator vt = new ValidatorTranslator(){
	    public boolean isValid(String s) {
	      try{Integer.parseInt(s); return true;}
	        catch(NumberFormatException pe){return false;}
	    }
	    public Object getValue(String s) {
	      try{return new Integer(Integer.parseInt(s));}
	        catch(NumberFormatException  e)
	          {return null;}
	    }
	  };
	  
	  PromptDialog pd = new PromptDialog("Enter int",message,"",vt);
	  return ((Integer)pd.getValue()).intValue();
  }

  
  
	/** 
	 * Returns the user's <code>int</code> response to the prompt message,
	 *   but only if it is in the range [low,high].
	 * 
	 * @param  message specifies the prompt displayed to the user
	 *         (augmented by the values of <code>low</code> and 
	 *          <code>high</code>)
	 * @param  low  specifies the minimum value the user can enter
	 * @param  high specifies the maximum value the user can enter
	 *
	 * @return the value entered by the user
	*/
  public static int forInt (String message, final int low, final int high)
  {
	  ValidatorTranslator vt = new ValidatorTranslator(){
	    public boolean isValid(String s) {
	      try{int temp = Integer.parseInt(s); return low <= temp && temp <= high;}
	        catch(NumberFormatException pe){return false;}
	    }
	    public Object getValue(String s) {
	      try{return new Integer(Integer.parseInt(s));}
	        catch(NumberFormatException  e)
	          {return null;}
	    }
	  };
	  
	  PromptDialog pd = new PromptDialog("Enter int",message,"",vt);
	  return ((Integer)pd.getValue()).intValue();
  }

  
  
	/** 
	 * Returns the user's <code>double</code> response to the prompt message.
	 * 
	 * @param  message specifies the prompt displayed to the user
	 *
	 * @return the value entered by the user
	*/
  public static double forDouble (String message)
  {
	  ValidatorTranslator vt = new ValidatorTranslator(){
	    public boolean isValid(String s) {
	      try{Double.parseDouble(s); return true;}
	        catch(NumberFormatException pe){return false;}
	    }
	    public Object getValue(String s) {
	      try{return new Double(Double.parseDouble(s));}
	        catch(NumberFormatException pe){return null;}
	    }
	  };
	  
	  PromptDialog pd = new PromptDialog("Enter double",message,"",vt);
	  return ((Double)pd.getValue()).doubleValue();
  }

  
  
	/** 
	 * Returns the user's <code>String</code> response to the prompt message.
	 * 
	 * @param  message specifies the prompt displayed to the user
	 *
	 * @return the value entered by the user
	*/
  public static String forString (String message)
  {
	  ValidatorTranslator vt = new ValidatorTranslator(){
	    public boolean isValid(String s) {
	      return true;
	    }
	    public Object getValue(String s) {
	      return s;
	    }
	  };
	  
	  PromptDialog pd = new PromptDialog("Enter char",message,"",vt);
	  return (String)pd.getValue();
  }


  
	/** 
	 * Returns the user's <code>boolean</code> response to the prompt message.
	 * 
	 * @param  message specifies the prompt displayed to the user
	 *
	 * @return the value entered by the user
	*/
  public static boolean forBoolean (String message)
  {
	  Object[] options = { "true", "false"};
	  int answer = JOptionPane.showOptionDialog(null,message,"Enter boolean",
	           JOptionPane.DEFAULT_OPTION, JOptionPane.QUESTION_MESSAGE,
	           null, options, options[0]);
	           
	  return answer == 0;
  }
  
  
  
	/** 
	 * Returns the user's <code>char</code> response to the prompt message.
	 * 
	 * @param  message specifies the prompt displayed to the user
	 *
	 * @return the value entered by the user
	*/
  public static char forChar (String message)
  {
	  ValidatorTranslator vt = new ValidatorTranslator(){
	    public boolean isValid(String s) {
	      return s.length()==1;
	    }
	    public Object getValue(String s) {
	      return new Character(s.charAt(0));
	    }
	  };
	  
	  PromptDialog pd = new PromptDialog("Enter char",message,"",vt);
	  return ((Character)pd.getValue()).charValue();
  }



	/** 
	 * Returns the user's <code>char</code> response to the prompt message,
	 *   but only if it is in the list of legal characters.
	 * 
	 * @param  message specifies the prompt displayed to the user
	 *         (augmented by the values in <code>legal</code>)
	 * @param  legal specifies the legal values the user can enter
	 *
	 * @return the value entered by the user
	*/
  public static char forChar (String message, final String legal)
  {
	  ValidatorTranslator vt = new ValidatorTranslator(){
	    public boolean isValid(String s) {
	      return s.length()==1 && legal.indexOf(s.charAt(0))!=-1;
	    }
	    public Object getValue(String s) {
	      return new Character(s.charAt(0));
	    }
	  };
	  
	  PromptDialog pd = new PromptDialog("Enter char",message,"",vt);
	  return ((Character)pd.getValue()).charValue();
  }



	/** 
	 * Returns the user's <code>BigInteger</code> response to the prompt
	 *   message.
	 * 
	 * @param  message specifies the prompt displayed to the user
	 *
	 * @return the value entered by the user
	*/
  public static BigInteger forBigInteger (String message)
  {
	  ValidatorTranslator vt = new ValidatorTranslator(){
	    public boolean isValid(String s) {
	      try{new BigInteger(s); return true;}
	        catch(NumberFormatException pe){return false;}
	    }
	    public Object getValue(String s) {
	      try{return new BigInteger(s);}
	        catch(NumberFormatException  e)
	          {return null;}
	    }
	  };
	  
	  PromptDialog pd = new PromptDialog("Enter BigInteger",message,"",vt);
	  return ((BigInteger)pd.getValue());
  }


	/** 
	 * Unit test for the methods in this class
	 * 
	 * @param  args ignored
	*/
	public static void main(String[] args)
	{
	  for (;;) {
	    System.out.println("\nLegend\n  i = int\n  d = double\n  b = boolean\n  c = char\n  s = string\n  B = BigInt\n  q = quit");
	    char c = Prompt.forChar("Enter type to prompt for","idbcsBq");
	    if (c == 'i')
	      System.out.println("Answer returned from prompt = " + GUIPrompt.forInt("Test: Enter int"));
	    else if (c == 'd')
	      System.out.println("Answer returned from prompt = " + GUIPrompt.forDouble("Test: Enter double"));
	    else if (c == 'b')
	      System.out.println("Answer returned from prompt = " + GUIPrompt.forBoolean("Test: Enter boolean"));
	    else if (c == 'c')
	      System.out.println("Answer returned from prompt = " + GUIPrompt.forChar("Test: Enter char"));
	    else if (c == 's')
	      System.out.println("Answer returned from prompt = " + GUIPrompt.forString("Test: Enter String"));
	    else if (c == 'B')
	      System.out.println("Answer returned from prompt = " + GUIPrompt.forBigInteger("Test: Enter BigInteger"));
	    else if (c == 'q')
	      break;
	  }
	}

  
}



//Helper interface and PromptDialog class

interface ValidatorTranslator
{
  boolean isValid (String s);
  Object getValue(String s);
}


class PromptDialog
{


	 
	PromptDialog (String title, String message,
	              String defaultText,
								ValidatorTranslator vT)
	{
	  //Store the only parameter needed by the listeners
	  this.vt = vT;
	  
	  //Construct the panel to contain all the information: both the
	  //  title and message, and the user's respose
	  final JPanel pp = new JPanel();
	  pp.setLayout(new BoxLayout(pp, BoxLayout.Y_AXIS));
	  pp.add(new JLabel (message));
	  answerText = new JTextField();
	  answerText.setText(defaultText);
	  pp.add(answerText);

	  //Put the panel in a JOptionPane (for formatting) and put the
	  //  JOptionPane in a JDialog: a heavyweight frame for displaying
	  final JOptionPane op = new JOptionPane(pp,JOptionPane.QUESTION_MESSAGE);
	  final JDialog     jd = new JDialog((Frame)null,title,true);
    final Toolkit     tk = Toolkit.getDefaultToolkit();
    final Dimension   d  = tk.getScreenSize();
	  jd.setLocation(d.width/4,d.height/4);
	  jd.toFront();
	 
	  //Handle own close, by generating a Property Change for op
	  //  so the JDialog doesn't really close unless the user enters
	  //  a valid value.
	  jd.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
	  jd.addWindowListener(new WindowAdapter()
	    {
	      public void windowClosing(WindowEvent we)
	      {op.setValue(answerText.getText());}
	    }
	  );  
	  
	  //Handle every PropertyChange to op (see window closing).
	  //When the user enters a valid value, set a field (the one
	  //  returned by <code>getValue</code>) and set the modal window
	  //  to invisible (the window is now done).
	  op.addPropertyChangeListener(new PropertyChangeListener()
	  {
	    public void propertyChange(PropertyChangeEvent e)
	    {
	      //Process it?
	      String prop = e.getPropertyName();
	      if (!jd.isVisible() ||
	           e.getSource() != op ||
	           !(prop.equals(JOptionPane.VALUE_PROPERTY) ||
	            prop.equals(JOptionPane.INPUT_VALUE_PROPERTY)))
	          return;
	          
	      //Yes! If not a bogus event, try to parse an integer, and
	      //if successful retur via jd.setVisible(false)
	      Object value = op.getValue();
	      if (value == JOptionPane.UNINITIALIZED_VALUE)
	        return;
	      if (vt.isValid(answerText.getText())){
	         //Valid: save the value and become invisible
	         op.setValue(vt.getValue(answerText.getText()));
	         jd.setVisible(false); // ends modal wait, process answer
	      }else{
	         //Not valid: beep and make sure a double button press, close,
	         //  or invalid text value is not really processed
	         Toolkit.getDefaultToolkit().beep();
	         op.setValue(JOptionPane.UNINITIALIZED_VALUE);
	      }
	   }
	 });
	   
	  //Whenever the JTextField is updated (by "enter"), activate the
	  //  ActionListener to remember the value; this triggers (as pressing
	  //  OK would) the PropertyChangeListener  above, to process the text.
	  answerText.addActionListener(new ActionListener()
	    {
	      public void actionPerformed(ActionEvent e)
	      {op.setValue(answerText.getText());}
	    }
	  );
	  
	  //Get the JDialogFrame ready to go and display it.
	  jd.setContentPane(op);
	  jd.pack();               //Pack window to preferred size
	  jd.setVisible(true);     //  and make is visible
	}


	Object getValue ()
	{return vt.getValue(answerText.getText());}

	JTextField          answerText;
	ValidatorTranslator vt;
}

