//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Class          : SimpleStack
//
// Author         : Richard E. Pattis
//                  Computer Science Department
//                  Carnegie Mellon University
//                  5000 Forbes Avenue
//                  Pittsburgh, PA 15213-3891
//                  e-mail: pattis@cs.cmu.edu
//
// Maintainer     : Author
//
//
// Description:
//
//   Objects constructed from the SimpleStack class act as collections of
// any objects. We can push objects into this collection and pop
// objects from the collection (according to the Last-In/First-Out
// ordering). Note that NO METHODS are called on these objects inside
// this class: we just move references around. 
//
// Future Plans   : None
//
// Program History:
//   9/25/01: R. Pattis - Operational for 15-100
//   8/10/04: R. Pattis - Javadoc comments added (added to course library)
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


package edu.cmu.cs.pattis.cs151xx;



/**
 * Objects constructed from the SimpleStack class act as collections of
 *   any references to objects.
 * We can push reference to objects into this collection and pop
 *   references to objects from the collection (according to the
 *   Last-In/First-Out ordering).
 * Note that NO METHODS are called on these objects inside this class:
 *   we just move references around.
*/ 
public class SimpleStack  { 


	/** 
	 * Constructs a <code>SimpleStack</code> object, specifying the
	 *   initial size for it (number of references it can store).
	 * Initially, the <code>SimpleStack</code> is empty.
	 *
	 * @param  initialSize specifies the number of references that the
	 *           <code>SimpleStack</code> can initially store.
	 *
	 * @throws IllegalArgumentException if <code>initialSize</b> is &lt 1
	*/
  public SimpleStack (int initialSize)
    throws IllegalArgumentException
  {
    if (initialSize < 1)
      throw new IllegalArgumentException("SimpleStack Constructor: initialSize("+initialSize+") < 1");

    stack = new Object[initialSize];
  }
  
  
  
	/** 
	 * Constructs a <code>SimpleStack</code> object, specifying the
	 *   initial size for it as 1.
	*/
  public SimpleStack ()
  {this(1);}
  
    
  
	/** 
	 * Make this <code>SimpleStack</code> empty.
	*/
  public void makeEmpty ()
  {
    for (int i=0; i<=top; i++)
      stack[i] = null;
    top = -1;
  }
  
  
  
	/** 
	 * Push a reference onto the top of this <code>SimpleStack</code>.
	 *
	 * @param o specifies the reference to push
	*/
  public void push (Object o)
  {
    if ( getSize() == stack.length)
        doubleLength();
        
    top++;
    stack[top] = o;        //or just stack[++top] = o;
  }
  
  
  
	/** 
	 * Returns the top reference on this <code>SimpleStack</code>, 
	 *   removing it in the process.
	 *
	 * @return the top reference on this <code>SimpleStack</code>,
	 *   removing it in the process
	 *
	 * @throws IllegalStateException if this <code>SimpleStack</code> is
	 *          empty
	*/
  public Object pop () throws IllegalStateException
  {
    if ( isEmpty() )
      throw new IllegalStateException("SimpleStack pop: Stack is empty");
    
    Object answer = stack[top];
    stack[top] = null;
    top--;
    return answer;
  }
    
    
  
  
	/** 
	 * Returns the top reference on this <code>SimpleStack</code>, 
	 *   without removing it in the process.
	 *
	 * @return the top reference on this <code>SimpleStack</code>,
	 *   without removing it in the process
	 *
	 * @throws IllegalStateException if this <code>SimpleStack</code> is
	 *          empty
	*/
  public Object peek () throws IllegalStateException
  {
    if ( isEmpty() )
      throw new IllegalStateException("SimpleStack peek: Stack is empty");
    
    return stack[top];  
  }
  
  
  
	/** 
	 * Returns whether or not this <code>SimpleStack</code> is empty
	 *   (stores no references).
	 *
	 * @return whether or not this <code>SimpleStack</code> is empty
	 *   (stores no references)
	*/
  public boolean isEmpty ()
  {return top == -1;}

  
  
	/** 
	 * Returns the number of reference this <code>SimpleStack</code>
	 *   stores (0 if empty).
	 *
	 * @return the number of reference this <code>SimpleStack</code>
	 *   stores (0 if empty)
	*/
  public int getSize ()
  {return top+1;}

  
  
	/** 
	 * Returns a <code>String<code> representation of the state of this 
	 *   <code>SimpleStack</code>.
	 * The <code>String</code> contains the value of <code>top</code>,
	 *   the length of the <code>stack</code> array, and all the references
	 *   stored in the <code>stack</code> array. 
	 *
	 * @return  a <code>String<code> representation of the state of this 
	 *  <code>SimpleStack</code>
	*/
  public String toString ()
  {
    String answer = "SimpleStack[top="+top+"/length="+stack.length;
    for (int i=0; i<=top; i++)
      answer += ";stack["+i+"]="+stack[i];
    answer +=  "]";
    
    return answer;
  }
  
  


  
	/** 
	 * Doubles the length of the <code>stack</code> array, while keeping
	 *   the same references in the same positions in it.
	 * This method is called only in <code>push</code> when the array is
	 *   filled.
	*/
  private void doubleLength ()
  {
    Object[] temp = new Object[stack.length*2];
    for (int i=0; i<stack.length; i++)
      temp[i] = stack[i];
    stack = temp;
  }


  
	/** 
	 * Stores the array of references between indexes <code>0</code> and
	 *   <code>top</code>.
	 * Its length is doubled when necessary, is always &gt <code>top</code>,
	 *   and never shrinks.
	*/
  private Object[] stack;
  
  
	/** 
	 * Stores the index of the top of the stack.
	 * In an empty stack it stores -1.
	 * There are always <code>top+1</code> references in the stack.
	*/
  private int      top = -1;
}




